export default {
    data() {
        return {
            config: {
                collection: 'admingroup',
                attributes: [
                    {
                        name: 'name',
                        display: 'Name',
                        sortable: true,
                        searchable: true
                    }
                ],
                routes: {
                    create: 'adminGroupCreate',
                    edit: 'adminGroupEdit'
                },
                typo: {
                    singular: 'group',
                    plural: 'groups',
                },
                itemsPerPage: 12
            },
            pagination: {
                current: 1,
                max: 1,
                total: 0
            },
            order: {
                attribute: false,
                direction: 'DESC'
            },
            collection: {
                list: [],
                total: 0
            },
            search: {
                text: false
            },
            fastMode: {
                active: false,
                create: false,
                model: null
            },
            selected: null,
            loading: []
        }
    },
    methods: {
        getList() {
            let
            vm = this,
            list = vm.$store.state.model[vm.collection].concat()
            
            if (vm.search.text) {
                let searchable_attributes = vm.attributes.filter((attribute) => {
                    return attribute.searchable
                }).map((attribute) => {
                    return attribute.name
                })

                list = list.filter((item) => {
                    for (let a in searchable_attributes) {
                        let prop = searchable_attributes[a]
                        if (item[prop].indexOf(vm.search.text) >= 0) {
                            return true
                        }
                    }

                    return false
                })
            }

            vm.pagination.total = list.length
            vm.pagination.max = parseInt( list.length / vm.config.itemsPerPage )
            if (vm.pagination.current > vm.pagination.max) {
                vm.pagination.current = vm.pagination.max < 1 ? 1 : vm.pagination.max
            }

            if (vm.order.attribute) {
                list.sort((itemA, itemB) => {
                    if ('function' === typeof sort_function) {
                        let res = sort_function(itemA, itemB)
                        return vm.order.direction === 'ASC' ? res : -res
                    }
                    if (itemA[vm.order.attribute] > itemB[vm.order.attribute]) {
                        return vm.order.direction === 'ASC' ? 1 : -1
                    }
                    if (itemA[vm.order.attribute] < itemB[vm.order.attribute]) {
                        return vm.order.direction === 'ASC' ? -1 : 1
                    }
                    return 0
                })
            }
            
            list = list.slice(vm.config.itemsPerPage * (vm.pagination.current - 1), vm.config.itemsPerPage * (vm.pagination.current - 1) + vm.config.itemsPerPage)

            if (vm.fastMode.active && vm.fastMode.create) {
                list.unshift({})
            }

            return list
        },
        sortByAttribute(attribute) {
            let vm = this

            if ('string' === typeof attribute) {
                attribute = vm.config.attributes.find(obj => obj.name === attribute)
            }

            if (!attribute) {
                return
            }

            if (vm.order.attribute == attribute.name) {
                vm.order.direction = vm.order.direction === 'ASC' ? 'DESC': 'ASC'
                vm.sort_function  = attribute.sort
            } else {
                vm.order.attribute = attribute.name
                vm.order.direction = 'ASC'
                vm.sort_function  = attribute.sort
            }
        },
        isSorted(attribute, direction) {
            let
            vm = this,
            is = vm.order.attribute == attribute.name

            if ('undefined' !== typeof direction) {
                is &= vm.order.direction == direction
            }

            return is
        },
        goToCreate() {
            this.$router.push({
                name: this.config.routes.create
            })
        },
        goToEdit(model) {
            let id = 'number' === typeof model ? model : model.id
            vm.$router.push({
                name: vm.config.routes.edit,
                params: { id }
            })
        },
        fastCreate() {
            vm.fastMode.model = {}
            vm.fastMode.create = true
            vm.fastMode.active = true
        },
        fastEdit(model) {
            vm.fastMode.model = JSON.parse(JSON.stringify(model))
            vm.fastMode.create = false
            vm.fastMode.active = true
        },
        fastClose() {
            this.fastMode.model = null
            this.fastMode.active = false
        },
        fastSave(model) {
            let vm = this
            vm.loading
            vm.$store.dispatch('model/update', {
                name: vm.collection,
                data: vm.fastMode.model,
                complete() {
                    if (vm.afterModelUpdate) {
                        vm.afterModelUpdate()
                    }
                    vm.fastMode.model = null
                    vm.fastMode.active = false
                }
            })
        },
        updateModel(data) {
            var vm = this
            if (vm.beforeModelUpdate) {
                vm.beforeModelUpdate()
            }
            vm.errors = {};
            vm.setLoading(data, true)
            vm.$store.dispatch('model/update', {
                name: vm.collection,
                data: data,
                success() {
                    if (vm.onModelUpdateSuccess) {
                        vm.onModelUpdateSuccess()
                    }
                },
                error(response, errors) {
                    vm.errors = errors
                    if (vm.onModelUpdateError) {
                        vm.onModelUpdateError()
                    }
                },
                complete() {
                    vm.setLoading(data, false)
                    if (vm.afterModelUpdate) {
                        vm.afterModelUpdate()
                    }
                }
            })
        },
        isLoading(model) {
            let id = 'number' === typeof model ? model : -1
            if ('object' === typeof model && model.id) {
                id = model.id
            }
            return this.loading.indexOf(id) >= 0
        },
        setLoading(model, is) {
            let id = 'number' === typeof model ? model : -1
            if ('object' === typeof model && model.id) {
                id = model.id
            }

            let index = this.loading.indexOf(id)

            if (is && index < 0) {
                this.loading.push(id)
            } else if (!is && index >= 0) {
                this.loading.splice(index, 1)
            }
        },
        select(model) {
            this.selected = model
        },
        deleteSelected() {
            var vm = this

            if (vm.beforeModelDelete) {
                vm.beforeModelDelete()
            }

            vm.setLoading(vm.selected.id, true)
            vm.$store.dispatch('model/delete', {
                name: vm.collection,
                id: vm.selected.id,
                success() {
                    if (vm.onModelDeleteSuccess) {
                        vm.onModelDeleteSuccess(vm.selected.id)
                    }
                    vm.selected.id = null
                },
                error() {
                    if (vm.onModelDeleteError) {
                        vm.onModelDeleteError(vm.selected.id)
                    }
                },
                complete() {
                    vm.setLoading(vm.selected.id, false)
                    if (vm.afterModelDelete) {
                        vm.afterModelDelete()
                    }
                }
            })
        }
    },
    created() {
        let vm = this

        vm.$store.dispatch('model/request', {
            collection: vm.collection
        })
    }
}