export default {
    data() {
        return {
            model: {},
            errors: {},
            loading: false
        }
    },
    methods: {
        submitModel() {
            this.updateModel(this.model)
        },
        submitForm(e) {
            let form = null

            if (e) {
                e.preventDefault()
                if (e.target && 'FORM' === e.target.tagName) {
                    form = e.target
                }
            }

            if (!form) {
                if ('FORM' === this.$el.tagName) {
                    form = this.$el
                } else {
                    form = this.$el.querySelector('form')
                }
            }

            if (!form) {
                throw new Error('Laravel-Vuex-Model Mixin Single: can\'t find form')
            }

            this.updateModel(new FormData(form))
        },
        updateModel(data) {
            var vm = this

            if (vm.loading) {
                return
            }

            if ('function' === typeof vm.beforeModelUpdate) {
                vm.beforeModelUpdate()
            }

            vm.errors = {};
            vm.loading = true
            vm.$store.dispatch('model/update', {
                name: vm.collection,
                data: data,
                success() {
                    if ('function' === typeof vm.onModelUpdateSuccess) {
                        vm.onModelUpdateSuccess()
                    }
                },
                error(response, errors) {
                    vm.errors = errors
                    if ('function' === typeof vm.onModelUpdateError) {
                        vm.onModelUpdateError()
                    }
                },
                complete() {
                    vm.loading = false
                    if ('function' === typeof vm.afterModelUpdate) {
                        vm.afterModelUpdate()
                    }
                }
            })
        },
        _name() {
            let
            args = Array.prototype.slice.call(arguments),
            name = args.shift()

            if (args.length) {
                name += '[' + args.join('][') + ']'
            }

            return name
        },
        _error() {
            let
            args = Array.prototype.slice.call(arguments),
            name = this._name.apply(this, args)

            return this.errors[name] ? this.errors[name] : false
        }
    },
    created() {
        var vm = this

        vm.$store.dispatch('model/request', {
            collection: vm.collection
        })

        if (vm.$route.params.id) {
            vm.loading = true
            vm.$store.dispatch('model/get', {
                collection: vm.collection,
                id: vm.$route.params.id,
                callback: (item) => {
                    vm.model = item
                    vm.loading = false
                    if (vm.onModelLoad) {
                        vm.onModelLoad(vm.model)
                    }
                }
            })
        }
    },
}