let
endpoints = {},
requests = {},
mutationTypes = [
    'set',
    'update'
]

let checkMutation = (mutation, name) => {
    return mutationTypes.indexOf(mutation.type) && name === mutation.payload.name
}

export default ({Vue, axios, echo}) => {
    return {
        create(context, { collection, endpoint, request }) {
            Vue.set(context.state, collection, [])
            endpoints[collection] = endpoint
    
            if (request) {
                context.dispatch('request', { collection })
            }
        },
        request(context, { collection }) {
            if ('undefined' !== typeof requests[collection]) {
                return
            }
    
            requests[collection] = true
    
            let next = (url) => {
                if (!url) {
                    return
                }
                
                axios.get(url)
                    .then(response => {
                        if ('undefined' === typeof response.data.current_page) {
                            return
                        }
    
                        if (response.data.next_page_url) {
                            next(response.data.next_page_url)
                        }
    
                        context.commit('update', {
                            collection,
                            model: response.data.data
                        })
                    })
            }
    
            axios.get('/api/'+endpoints[collection])
                .then(response => {
                    if (response.data instanceof Array) {
                        context.commit('set', {
                            collection,
                            models: response.data
                        })
                    } else if ('undefined' !== typeof response.data.current_page) {
                        if (response.data.next_page_url) {
                            next(response.data.next_page_url)
                        }
    
                        context.commit('set', {
                            collection: collection,
                            models    : response.data.data
                        })
                    }
                })
    
            context.dispatch('echo', { collection })
        },
        requestAll(context) {
            Object.keys(endpoints).forEach((collection) => {
                context.dispatch('request', { collection })
            })
        },
        echo(context, { collection }) {
            if (echo) {
                echo.channel(collection)
                    .listen('ModelUpdated', (e) => {
                        context.commit('update', {
                            collection,
                            model: e.model
                        })
                    })
                    .listen('ModelDeleted', (e) => {
                        context.commit('delete', {
                            collection,
                            id: e.id
                        })
                    })
            }
        },
        get(context, { collection, id, callback }) {
            if (typeof id === 'undefined') {
                callback(null)
            }
    
            if (typeof context.state[collection] !== 'undefined') {
                let item = context.state[collection].find(item => id == item.id)
                if (item) {
                    callback(item)
                } else {
                    axios.get('/api/'+endpoints[collection]+'/'+id)
                        .then(response => {
                            context.commit('update', {
                                collection: collection,
                                model: response.data
                            })
                            callback(response.data)
                        })
                }
                return
            }
    
            this.subscribe((mutation, state) => {
                if (mutationTypes.indexOf(mutation.type) >= 0 && collection === mutation.payload.name) {
                    let item = context.state[collection].find(item => id == item.id)
                    if (item) {
                        callback(item)
                    }
                }
            })
        },
        update(context, { collection, data, success, error, complete }) {
            let
            endpoint = endpoints[collection],
            url = '/api/'+endpoint
    
            if (data instanceof FormData) {
                if (data.get('id')) {
                    data.append('_method', 'PUT')
                    url = '/api/'+endpoint+'/'+data.get('id')
                }
            } else {
                if (typeof data.id !== 'undefined') {
                    data['_method'] = 'PUT'
                    url = '/api/'+endpoint+'/'+data.id
                }
            }
    
            axios.post(url, data)
                .then(response => {
                    context.commit('update', {
                        collection: collection,
                        model: response.data
                    })
                    if (typeof success === 'function') {
                        success(response)
                    }
                    if (typeof complete === 'function') {
                        complete(response)
                    }
                }).catch(response => {
                    if (typeof error === 'function' || typeof complete === 'function') {
                        let errors = {}
                        for (let key in response.response.data) {
                            let errorKey = key
                            
                            if (errorKey.indexOf('.') >= 0) {
                                let
                                split = errorKey.split('.'),
                                first = split.shift()
    
                                split = split.map((str) => {
                                    return '['+str+']'
                                })
                                
                                split.unshift(first)
                                errorKey = split.join('')
                            }
    
                            errors[errorKey] = response.response.data[key]
                            if (errors[errorKey] instanceof Array) {
                                errors[errorKey] = errors[errorKey].shift()
                            }
                        }
                        if (typeof error === 'function') {
                            error(response, errors)
                        }
                        if (typeof complete === 'function') {
                            complete(response)
                        }
                    }
                })
        },
        delete(context, { collection, id, success, error, complete }) {
            let
            endpoint = endpoints[collection],
            data = new FormData()
            data.append('_method', 'DELETE')
    
            axios.post('/api/'+endpoint+'/'+id, data)
                .then(response => {
                    context.commit('delete', {collection, id})
                    if (typeof success === 'function') {
                        success(response)
                    }
                    if (typeof complete === 'function') {
                        complete(response)
                    }
                }).catch(response => {
                    if (typeof error === 'function') {
                        error(response)
                    }
                    if (typeof complete === 'function') {
                        complete(response)
                    }
                })
        }
    }
}